# /// script
# requires-python = ">=3.9"
# dependencies = [
#     "shipyard-dbt",
# ]
# ///
import os
import sys

from shipyard_dbt import DbtClient


def main():
    sys.exit(
        DbtClient(
            access_token=os.getenv("DBT_API_KEY"),
            account_id=os.getenv("DBT_ACCOUNT_ID"),
        ).connect()
    )


if __name__ == "__main__":
    main()



import os
from shipyard_dbt import DbtClient
from dotenv import load_dotenv, find_dotenv

load_dotenv(find_dotenv())


def conn_helper(client: DbtClient):
    conn = client.connect()
    if conn == 0:
        return 0
    else:
        client.logger.error("Could not connect to DBT")
        return 1


def test_good_connection():
    dbt = DbtClient(
        access_token=os.getenv("DBT_API_KEY"), account_id=os.getenv("DBT_ACCOUNT_ID")
    )
    assert dbt.connect() == 0


def test_bad_token():
    dbt = DbtClient(access_token="bad_token", account_id=os.getenv("DBT_ACCOUNT_ID"))
    assert conn_helper(dbt) == 1


def test_bad_account():
    dbt = DbtClient(access_token=os.getenv("DBT_API_KEY"), account_id="bad_account")
    assert conn_helper(dbt) == 1



import os
import platform

import requests
from shipyard_templates import Etl, ShipyardLogger, ExitCodeException
from shipyard_templates.etl import BadRequestError, UnauthorizedError, UnknownError

logger = ShipyardLogger.get_logger()

CHUNK_SIZE = 16 * 1024 * 1024
EXIT_CODE_INVALID_RESOURCE = 101


class InvalidResourceError(ExitCodeException):
    def __init__(self, message):
        super().__init__(message, EXIT_CODE_INVALID_RESOURCE)


class DbtClient(Etl):
    EXIT_CODE_STATUS_INCOMPLETE = 110
    EXIT_CODE_FINAL_STATUS_ERRORED = 111
    EXIT_CODE_FINAL_STATUS_CANCELLED = 112

    def __init__(self, access_token: str, account_id: str):
        self.access_token = access_token
        self.account_id = account_id
        self.headers = {"Authorization": f"Bearer {access_token}"}
        self.account_url = f"https://cloud.getdbt.com/api/v2/accounts/{account_id}/"
        self.base_url = "https://cloud.getdbt.com/api/v2/"

        super().__init__(
            access_token=access_token,
            account_id=account_id,
            headers=self.headers,
            base_url=self.base_url,
            account_url=self.account_url,
        )

    def _request(self, url, data=None, params=None, method="GET"):
        request_details = {"headers": self.headers}
        if data:
            request_details["data"] = data
        if params:
            request_details["params"] = params

        try:
            response = requests.request(method, url, **request_details)
            if response.ok:
                return response.json()
            if response.status_code == 401:
                raise UnauthorizedError(response.text)

            if response.status_code == 404:
                raise InvalidResourceError(
                    f"Resource not found. Check to make sure that the URL was typed correctly.\n{response.json()}"
                )

            response.raise_for_status()

        except requests.exceptions.HTTPError as eh:
            raise BadRequestError(f"URL returned an HTTP Error.\n{eh}") from eh
        except requests.exceptions.ConnectionError as ec:
            raise UnknownError(ec)
        except requests.exceptions.Timeout as et:
            raise ExitCodeException(
                f"Timed out while connecting to the URL.\n{et}", self.TIMEOUT
            ) from et
        except requests.exceptions.RequestException as e:
            raise BadRequestError(e) from e

    def trigger_sync(self, job_id):
        logger.info(f"Triggering sync for job {job_id} on account {self.account_id}")

        source_information = (
            f'Fleet ID: {os.environ.get("SHIPYARD_FLEET_ID")} Vessel ID: {os.environ.get("SHIPYARD_VESSEL_ID")} Log ID: {os.environ.get("SHIPYARD_LOG_ID")}'
            if os.environ.get("USER") == "shipyard"
            else f"Run on {platform.platform()}"
        )

        response = self._request(
            f"{self.account_url}/jobs/{job_id}/run/",
            data={"cause": f"Run by {os.environ['USER']} - {source_information}"},
            method="POST",
        )
        logger.debug(f"Response from the API: \n{response}")
        return response

    def determine_sync_status(self, run_id):
        run_details = self.get_run_details(run_id)
        run_id = run_details["data"]["id"]
        if run_details["data"]["is_complete"]:
            if run_details["data"]["is_error"]:
                logger.info(f"dbt Cloud reports that the run {run_id} erred.")
                return self.EXIT_CODE_FINAL_STATUS_ERRORED
            elif run_details["data"]["is_cancelled"]:
                logger.info(f"dbt Cloud reports that run {run_id} was cancelled.")
                return self.EXIT_CODE_FINAL_STATUS_CANCELLED
            else:
                logger.info(f"dbt Cloud reports that run {run_id} was successful.")
                return self.EXIT_CODE_FINAL_STATUS_COMPLETED
        else:
            logger.info(
                f"dbt Cloud reports that the run {run_id} is not yet completed."
            )
            return self.EXIT_CODE_STATUS_INCOMPLETE

        #     return exit_code

    def connect(self):
        try:
            response = requests.get(self.account_url, headers=self.headers)
            if response.status_code == 200:
                logger.info("Successfully connected to DBT")
                return 0
            else:
                logger.error("Could not connect to DBT")
                return 1
        except Exception as e:
            logger.error(f"Could not connect to DBT due to {e}")
            return 1

    def _determine_connection_status(self, response):
        status_code = response["status"]["code"]
        user_message = response["status"]["user_message"]
        if status_code == 401:
            if "Invalid token" in user_message:
                raise UnauthorizedError(
                    f"The Service Token provided was invalid. Check to make sure there are no typos or "
                    f"preceding/trailing spaces. dbt API says: {user_message}",
                )
            else:
                raise UnknownError(
                    f"An unknown error occurred with a status code of {status_code}. dbt API says: {user_message}",
                )
        if status_code == 404:
            raise InvalidResourceError(
                f"The Account ID, Job ID, or Run ID provided was either invalid or your API Key doesn't"
                f"have access to it. Check to make sure there are no typos or preceding/trailing spaces. "
                f"dbt API says: {user_message}"
            )

    def get_run_details(self, run_id):
        logger.info(f"Grabbing run details for run {run_id}.")

        response = self._request(
            url=f"{self.account_url}/runs/{run_id}/",
            params={"include_related": "['run_steps','debug_logs']"},
        )
        logger.debug(f"Response from the API: \n{response}")
        return response

    def get_artifact_details(self, run_id):
        logger.info(f"Grabbing artifact details for run {run_id}")
        return self._request(f"{self.account_url}/runs/{run_id}/artifacts/")

    def download_artifact(self, run_id, artifact_name, destination_folder):
        get_artifact_details_url = (
            f"{self.account_url}/runs/{run_id}/artifacts/{artifact_name}"
        )
        artifact_file_name = os.path.basename(artifact_name)
        artifact_folder = os.path.dirname(artifact_name)

        destination_fullpath = os.path.join(destination_folder, artifact_folder)
        os.makedirs(destination_fullpath, exist_ok=True)

        filename = os.path.join(destination_fullpath, artifact_file_name)

        try:
            with requests.get(
                get_artifact_details_url, headers=self.headers, stream=True
            ) as r:
                r.raise_for_status()
                with open(filename, "wb") as f:
                    for chunk in r.iter_content(chunk_size=CHUNK_SIZE):
                        f.write(chunk)
            logger.info(f"Successfully downloaded file {get_artifact_details_url}")
        except Exception as e:
            raise UnknownError(
                f"Failed to download file {get_artifact_details_url}",
            ) from e



import argparse
import sys
import time

import shipyard_bp_utils as shipyard
from shipyard_bp_utils.artifacts import Artifact
from shipyard_templates import ShipyardLogger, Etl, ExitCodeException

from shipyard_dbt import DbtClient

logger = ShipyardLogger.get_logger()


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--api-key", dest="api_key", required=True)
    parser.add_argument("--account-id", dest="account_id", required=True)
    parser.add_argument("--job-id", dest="job_id", required=True)
    parser.add_argument(
        "--download-artifacts",
        dest="download_artifacts",
        default="TRUE",
        required=False,
    )
    parser.add_argument(
        "--download-logs",
        dest="download_logs",
        default="TRUE",
        required=False,
    ),

    return parser.parse_args()


def main():
    exit_code = 0
    try:
        artifact = Artifact(
            "dbtcloud"
        )  # Note: dbt has a concept of artifact as well. This is a different artifact.

        args = get_args()
        download_artifacts = shipyard.args.convert_to_boolean(args.download_artifacts)
        download_logs = shipyard.args.convert_to_boolean(args.download_logs)

        job_id = args.job_id

        client = DbtClient(args.api_key, args.account_id)

        job_run_response = client.trigger_sync(job_id)
        artifact.responses.write_json(f"job_{job_id}_response", job_run_response)

        run_id = job_run_response["data"]["id"]
        artifact.variables.create_pickle("run_id", run_id)

        is_complete = False
        while not is_complete:
            run_details_response = client.get_run_details(run_id)
            is_complete = run_details_response["data"]["is_complete"]
            if not is_complete:
                logger.info(
                    f"Run {run_id} is not complete. Waiting 60 seconds and trying again."
                )
                time.sleep(60)
            else:
                exit_code = client.determine_sync_status(run_id)

        if download_logs:
            number_of_steps = len(run_details_response["data"]["run_steps"])
            if number_of_steps > 0:
                debug_log_name = shipyard.files.combine_folder_and_file_name(
                    artifact.logs.path, "dbt.log"
                )
                output_log_name = shipyard.files.combine_folder_and_file_name(
                    artifact.logs.path, "dbt_console_output.txt"
                )

                logger.info(f"Downloading logs for {number_of_steps} steps")
                for index, step in enumerate(
                    run_details_response["data"]["run_steps"], start=1
                ):
                    step_id = step["id"]
                    logger.info(
                        f"Grabbing step details for step {step_id} ({index} of {number_of_steps})"
                    )
                    artifact.responses.write_json(f"step_{step_id}_response", step)
                    if debug_log := step.get("debug_logs"):
                        with open(debug_log_name, "a") as debug_file:
                            debug_file.write(debug_log)
                    else:
                        logger.warning(
                            f"No debug logs for step {step_id}. Skipping ..."
                        )
                    if log := step.get("logs"):
                        with open(output_log_name, "a") as log_file:
                            log_file.write(log)
                    else:
                        logger.warning(f"No logs for step {step_id}. Skipping ...")

                logger.info(f"Successfully wrote logs to {output_log_name}")
                logger.info(f"Successfully wrote debug_logs to {debug_log_name}")
            elif number_of_steps == 0:
                logger.info(f"No logs to download for run {run_id}")

        if download_artifacts:
            dbt_artifact_response = client.get_artifact_details(run_id)
            artifact.responses.write_json(
                f"artifacts_{run_id}_response", dbt_artifact_response
            )
            dbt_artifacts = dbt_artifact_response["data"]

            if dbt_artifacts:
                artifact_path = artifact.create_folder("artifacts")
                number_of_artifacts = len(dbt_artifacts)
                logger.info(f"Downloading {number_of_artifacts} artifacts")
                for index, dbt_artifact in enumerate(dbt_artifacts, start=1):
                    logger.info(f"Downloading file {index} of {number_of_artifacts}")
                    client.download_artifact(run_id, dbt_artifact, artifact_path)
            else:
                logger.info("No artifacts exist for this run.")

        sys.exit(exit_code)
    except ExitCodeException as e:
        logger.error(e)
        sys.exit(e.exit_code)
    except Exception as e:
        logger.error(f"An unknown error occurred: {e}")
        sys.exit(Etl.EXIT_CODE_UNKNOWN_ERROR)


if __name__ == "__main__":
    main()
